# -*- coding: utf-8 -*-
import pytest


@pytest.mark.parametrize("packages,expected", [
    (['pkg1', 'pkg2', 'pkg3'], ({'pkg1', 'pkg3'}, {'pkg2'})),
])
def test_classify_installed_or_not(monkeypatch, packages, expected):
    def mockreturn():
        return {'pkg2'}
    from pit import utils
    monkeypatch.setattr(utils, 'get_installed_package_set', mockreturn)
    from pit.utils import classify_installed_or_not

    rv = classify_installed_or_not(packages)
    assert rv == expected


def test_re_edit_requirements():
    from pit.utils import re_edit_requirements
    before = [
        "# requirements.in\n",
        "flask\n",
        "pytest\n",
        "\n",
    ]
    after = """# requirements.in
pytest

"""
    content = re_edit_requirements(before, ['flask'])
    assert content == after



# -*- coding: utf-8 -*-
from __future__ import division, print_function, absolute_import, unicode_literals

import click


def info(message):
    """Display Information message

    :param message: Information message

    """
    click.echo(message)


def warn(message):
    """Display Warning message

    :param message: Warning message

    """
    click.echo(click.style(message, fg='yellow'))


def err(message):
    """Display Error message

    :param message: Error message

    """
    click.echo(click.style(message, fg='red'))



# -*- coding: utf-8 -*-
from __future__ import division, print_function, absolute_import, unicode_literals

import click


class AliasedGroup(click.Group):

    def _get_matched_commands(self, ctx, cmd):
        return [x for x in self.list_commands(ctx) if x.startswith(cmd)]

    def _get_command(self, ctx, cmd_name):
        return click.Group.get_command(self, ctx, cmd_name)

    def get_command(self, ctx, cmd_name):
        """Get command by aliased command name"""

        cmd = self._get_command(ctx, cmd_name)
        if cmd is not None:
            return cmd
        matches = self._get_matched_commands(ctx, cmd_name)
        if not matches:
            return None
        elif len(matches) == 1:
            return self._get_command(ctx, matches[0])
        ctx.fail('Too many matches: {}'.format(', '.join(sorted(matches))))



# -*- coding: utf-8 -*-
from click.testing import CliRunner


def test_version():
    from pit.cli import cli
    from pit import __version__
    runner = CliRunner()
    result = runner.invoke(cli, ['--version'])
    assert "snake-pit version: {}".format(__version__) in result.output


def test_install():
    from pit.cli import cli
    runner = CliRunner()
    result = runner.invoke(cli, ['i'])
    assert "You must give at least one requirement to install" in result.output


def test_uninstall():
    from pit.cli import cli
    runner = CliRunner()
    result = runner.invoke(cli, ['u'])
    assert "Do you want to continue?" in result.output



# -*- coding: utf-8 -*-
import pytest


@pytest.mark.parametrize("cmd,expected_output", [
    ('install', "I'm install"),
    ('i', "I'm install"),
    ('uninstall', "I'm uninstall"),
    ('un', "I'm uninstall"),
    ('u', "Too many matches: uninstall, update"),
    ('XXX', 'No such command'),
])
def test_aliased_group(cmd, expected_output):
    import click
    from click.testing import CliRunner
    from pit.groups import AliasedGroup

    @click.group(cls=AliasedGroup)
    def pit():
        click.echo("I'm pit")

    @pit.command()
    def install():
        click.echo("I'm install")

    @pit.command()
    def uninstall():
        click.echo("I'm uninstall")

    @pit.command()
    def update():
        click.echo("I'm update")

    runner = CliRunner()
    result = runner.invoke(pit, [cmd])
    assert expected_output in result.output



# -*- coding: utf-8 -*-
import re
import ast

from setuptools import setup, find_packages


_version_re = re.compile(r'__version__\s+=\s+(.*)')


with open('pit/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))


try:
    long_description = open("README.rst").read()
except IOError:
    long_description = ""

setup(
    name="snake-pit",
    version=version,
    description=(
        "Depending on the installation or uninstall packages, "
        "and then edit the requirements file."
    ),
    license="MIT",
    author="kk6",
    author_email="hiro.ashiya@gmail.com",
    url="https://github.com/kk6/snake-pit",
    packages=find_packages(exclude=['tests*']),
    install_requires=[
        "Click",
    ],
    tests_require=['pytest'],
    long_description=long_description,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: 3.5",
        "Topic :: Utilities"
    ],
    entry_points="""
        [console_scripts]
        pit = pit.__main__:cli
    """
)



# -*- coding: utf-8 -*-
from __future__ import division, print_function, absolute_import, unicode_literals

import pip
import click

from . import echoes
from .groups import AliasedGroup
from .utils import (
    get_installed_package_set,
    classify_installed_or_not,
    re_edit_requirements,
    get_dependencies,
    print_version,
)


@click.group(cls=AliasedGroup)
@click.option('--version', '-V', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True)
def cli():
    """Depending on management packages, and then edit the requirements file.
    """


@cli.command()
@click.argument('packages', nargs=-1)
@click.option(
    '--requirement', '-r', default='requirements.in', type=click.File('a'),
    help="Append package names to the given requirement file."
)
def install(packages, requirement):
    """Install packages and write requirements file.

    To append package names were installed in requirements file,
    if the 'pip install' was successful.

    :param packages: Install packages.
    :param requirement: Output destination of package names.

    """
    if not packages:
        echoes.warn("You must give at least one requirement to install"
                    "(see 'pit --help')")
        return

    will_install, need_upgrade = classify_installed_or_not(packages)

    if need_upgrade:
        echoes.info(
            "Following packages installed. "
            "(use pip install --upgrade to upgrade): {}".format(", ".join(need_upgrade)))

    if not will_install:
        echoes.warn("There is no installable packages a new.")
        return

    raised = pip.main(["install"] + [pkg for pkg in will_install])
    if raised:
        return

    requirement.write('\n'.join(packages) + '\n')
    msg = "Append the following packages in {requirement}: {packages}"
    echoes.info(msg.format(requirement=requirement.name,
                           packages=", ".join(will_install)))


@cli.command()
@click.argument('packages', nargs=-1)
@click.option(
    '--requirement', '-r', default='requirements.in', type=click.File('r'),
    help="Remove package names from the given requirement file."
)
@click.confirmation_option(help="Are you sure you want to uninstall these packages?")
def uninstall(packages, requirement):
    """Uninstall packages and remove from requirements file.

    To Remove uninstalled packages from requirements file,
    if the 'pip uninstall' was successful.

    :param packages: Uninstall packages.
    :param requirement: Output destination of left package names.

    """
    if not packages:
        echoes.err(
            "You must give at least one requirement to uninstall"
            "(see 'pit --help')"
        )
        return

    uninstalled_packages = []
    for pkg in packages:
        if pkg in uninstalled_packages:
            # Already installed
            continue
        if pkg not in get_installed_package_set():
            echoes.err("{} is not installed".format(pkg))
            continue
        if pip.main(["uninstall"] + ['-y'] + get_dependencies(pkg)):
            # Uninstall failed.
            continue
        else:
            uninstalled_packages.append(pkg)

    if not uninstalled_packages:
        return

    content = re_edit_requirements(requirement.readlines(), packages)
    with open(requirement.name, 'w') as f:
        f.write(content)
    msg = "Remove the following packages from {requirement}: {packages}"
    echoes.info(msg.format(requirement=requirement.name,
                           packages=", ".join(uninstalled_packages)))



# -*- coding: utf-8 -*-
from __future__ import division, print_function, absolute_import, unicode_literals

import re

from pkg_resources import working_set
import pip

from . import __version__
from . import echoes


def print_version(ctx, param, value):
    """Print version info."""

    if not value or ctx.resilient_parsing:
        return
    echoes.info('snake-pit version: {}'.format(__version__))
    ctx.exit()


def get_installed_package_set():
    """Returns installed packages

    :return: Installed packages.
    :rtype: set

    """
    return {p.key for p in pip.get_installed_distributions()}


def classify_installed_or_not(packages):
    """Classify packages installed or not

    :param packages: Passed packages by user.
    :return: Install candidates and installed packages.
    :rtype: tuple

    """
    installed = get_installed_package_set()
    will_install = set(packages) - installed
    need_upgrade = set(packages) & installed
    return will_install, need_upgrade


def re_edit_requirements(lines, will_remove):
    """Re-Edit requirements file

    :param lines: requirement.readlines()'s return value.
    :param will_remove: Will be removed packages.
    :return: The contents of the file after re-editing.
    :rtype: str

    """
    pattern = re.compile('^[\w0-9\-.]+')
    re_editing = []
    for line in lines:
        matched = pattern.match(line)
        if not matched or matched.group().lower() not in will_remove:
            re_editing.append(line)
    if re_editing and re_editing[-1] != '\n':
        re_editing.append('\n')
    return ''.join(re_editing)


def get_dependency_tree(package):
    """Returns dependency tree by nested list

    :param package: Top level package.
    :return: Dependency tree.
    :rtype: list

    """
    tree = [package]
    top_pkg = working_set.by_key[package]
    for p in top_pkg.requires():
        tree.append(get_dependency_tree(p.key))
    return tree


def get_dependencies(package):
    """Returns dependencies by flat list

    :param package: Top level package.
    :return: Dependencies
    :rtype: list

    """
    dependencies = [package]
    top_pkg = working_set.by_key[package]
    for p in top_pkg.requires():
        dependencies.extend(get_dependencies(p.key))
    return dependencies
